-- ###
-- Most from here is from the AceComm-3.0 lib as it not works with battle.net messages
-- EVENTS:
-- "Sender_MessageSend"	(senderHandler, playerTemplate, callbackValue)
-- ###

local AddonName, GGL = ...

local ADDON_CHANNEL = "GGL-SYNC-"..GGL.SYNC_VERSION
local ADDON_CHANNEL_VERSION_CHECK = "GGL_VCHECK"

local Sender = {}
GGL.Sender = Sender

Sender.callbacks = LibStub("CallbackHandler-1.0"):New(Sender, "CH_RegisterEvent", "CH_UnregisterEvent", "CH_UnregisterAllEvents")
local LibDeflate = LibStub:GetLibrary("LibDeflate")


-- lua
local assert, type = assert, type
local match, strsub = string.match, strsub
local tinsert, tconcat = table.insert, table.concat

-- wow
local RegisterAddonMessagePrefix = C_ChatInfo.RegisterAddonMessagePrefix
local BNSendGameData, SendAddonMessage = BNSendGameData, C_ChatInfo.SendAddonMessage
local C_TimerAfter = C_Timer.After

-- EventFrame
local EventFrame = CreateFrame("FRAME")
local  function OnEvent(self, event, ...)
	Sender[event](Sender, ...)
end
EventFrame:SetScript("OnEvent", OnEvent)

-- ###########################
-- Local
-- ###########################
local PLAYER_WOW, PLAYER_BN

local MSG_MULTI_FIRST = "\001"
local MSG_MULTI_NEXT  = "\002"
local MSG_MULTI_LAST  = "\003"
local MSG_ESCAPE = "\004"

local MSG_PREFIX_SPLIT = "\005"
local MSG_SEND_FORMAT, MSG_GET_FORMAT = "%s"..MSG_PREFIX_SPLIT, "^(.*)"..MSG_PREFIX_SPLIT.."(.*)"

local SEND_DELAY = 0.1

local PrefixRegister = {}
local Workflow = {}

-- ###########################
-- Init
-- ###########################
local function Init()
	PLAYER_WOW = GGL.Templates.WoWPlayer
	PLAYER_BN = GGL.Templates.BNWoWPlayer

	RegisterAddonMessagePrefix(ADDON_CHANNEL)
	RegisterAddonMessagePrefix(ADDON_CHANNEL_VERSION_CHECK)

	EventFrame:RegisterEvent("BN_CHAT_MSG_ADDON")
	EventFrame:RegisterEvent("CHAT_MSG_ADDON")
end
GGL:AddInitFunc(Init)

-- ###########################
-- Sender
-- ###########################
local function SenderHelpFunc(isBNMessage, message, target)
	if isBNMessage then
		BNSendGameData(target, ADDON_CHANNEL, message)
	else
		SendAddonMessage(ADDON_CHANNEL, message, "WHISPER", target)
	end
end

local function SendMsg(isBNMessage, message, target, playerTemplate)
	if playerTemplate and playerTemplate.CanSendMessageNow and not playerTemplate:CanSendMessageNow() then
		return GGL:AddIntoErrorLog("SendMsg", playerTemplate and {playerTemplate.syncID, playerTemplate:CanSendMessageNowRejectMessage()} or "unknown player")
	end

	local maxTextLen
	if isBNMessage then
		if not target or target < 1 then return end
		maxTextLen = 4078
	else
		if not target then return end
		maxTextLen = 255
	end
	local textLen = #message

	-- This is mostly from AceComm
	local forceMultipart
	if match(message, "^[\001-\009]") then -- 4.1+: see if the first character is a control character
		-- we need to escape the first character with a \004
		if textLen+1 > maxTextLen then	-- would we go over the size limit?
			forceMultipart = true	-- just make it multipart, no escape problems then
		else
			message = "\004" .. message
		end
	end

	if not forceMultipart and textLen <= maxTextLen then
		-- fits all in one message
		SenderHelpFunc(isBNMessage, message, target)
	else
		maxTextLen = maxTextLen - 1	-- 1 extra byte for part indicator in prefix(4.0)/start of message(4.1)

		-- first part
		local chunk = strsub(message, 1, maxTextLen)
		SenderHelpFunc(isBNMessage, MSG_MULTI_FIRST..chunk, target)

		-- continuation
		local pos = 1+maxTextLen

		while pos+maxTextLen <= textLen do
			chunk = strsub(message, pos, pos+maxTextLen-1)
			SenderHelpFunc(isBNMessage, MSG_MULTI_NEXT..chunk, target)
			pos = pos + maxTextLen
		end

		-- final part
		chunk = strsub(message, pos)
		SenderHelpFunc(isBNMessage, MSG_MULTI_LAST..chunk, target)
	end
end

function Sender:SendBNMessageInstant(presenceID, message, playerTemplate)
	if not presenceID or not message then return end
	SendMsg(true, message, presenceID, playerTemplate)
end

function Sender:SendAddonMessageInstant(target, message, playerTemplate)
	if not target or not message then return end
	SendMsg(false, message, target, playerTemplate)
end

-- ###########################
-- DelaySender
-- ###########################
local WorkflowPosition, WorkflowSize = 1,0

local function ProcessWorkflow()
	-- reset workflow if its empty
	if WorkflowPosition > WorkflowSize then
		WorkflowPosition, WorkflowSize = 1,0
		return
	end
	-- get wf item
	local wf = Workflow[WorkflowPosition]
	if not wf then return end

	-- skip offline players
	if wf[6] and wf[6].CanSendMessageNow and not wf[6]:CanSendMessageNow() then
		GGL:AddIntoErrorLog("ProcessWorkflow", wf[6] and {wf[6].syncID, wf[6]:CanSendMessageNowRejectMessage()} or "unknown player")
		Workflow[WorkflowPosition] = nil
		WorkflowPosition = WorkflowPosition + 1
		return ProcessWorkflow()
	end

	-- push msg
	if wf[4].registered then
		SendMsg(wf[1], wf[2], wf[3], wf[6])
	else
		Workflow[WorkflowPosition] = nil
		WorkflowPosition = WorkflowPosition + 1
		return ProcessWorkflow()
	end

	-- fire callback
	if wf[5] then
		wf[4]:FireCallbackEvent("Sender_MessageSend", wf[4], wf[6], wf[5])
	end

	-- remove from workflow and set new position
	Workflow[WorkflowPosition] = nil
	WorkflowPosition = WorkflowPosition + 1

	-- delay next msg
	C_TimerAfter(SEND_DELAY, ProcessWorkflow)
end

local function AddIntoWorkflow(isBNMessage, target, message, senderHandler, callbackValue, playerTemplate)
	WorkflowSize = WorkflowSize + 1
	if senderHandler and callbackValue and playerTemplate then
		Workflow[WorkflowSize] = {isBNMessage, message, target, senderHandler, callbackValue, playerTemplate}
	else
		Workflow[WorkflowSize] = {isBNMessage, message, target, senderHandler, nil, playerTemplate}
	end
	if WorkflowSize == 1 then
		ProcessWorkflow()
	end
end

function Sender:SendBNMessage(presenceID, message, sendInstant, senderHandler, callbackValue, playerTemplate)
	if not presenceID or not message then return end
	if sendInstant then
		SendMsg(true, message, presenceID, playerTemplate)
		if callbackValue and playerTemplate then
			senderHandler:FireCallbackEvent("Sender_MessageSend", senderHandler, playerTemplate, callbackValue)
		end
	else
		AddIntoWorkflow(true, presenceID, message, senderHandler, callbackValue, playerTemplate)
	end
end

function Sender:SendWowMessage(target, message, sendInstant, senderHandler, callbackValue, playerTemplate)
	if not target or not message then return end
	if sendInstant then
		SendMsg(false, message, target, playerTemplate)
		if callbackValue and playerTemplate then
			senderHandler:FireCallbackEvent("Sender_MessageSend", senderHandler, playerTemplate, callbackValue)
		end
	else
		AddIntoWorkflow(false, target, message, senderHandler, callbackValue, playerTemplate)
	end
end

-- ###########################
-- Register
-- cb
-- MessageSend(senderHandler, playerTemplate, callbackValue)
-- ###########################
local SENDER_PROTO = {
	["SendBNMessage"] = function(self, playerTemplate, message, sendInstant, callbackValue)
		if not playerTemplate:CanSendMessage(self.ignoreGGL) then return end
		Sender:SendBNMessage(playerTemplate.sender, self.fullPrefix..message, sendInstant, self, callbackValue, playerTemplate)
	end,
	["SendWoWMessage"] = function(self, playerTemplate, message, sendInstant, callbackValue)
		if not playerTemplate:CanSendMessage(self.ignoreGGL) then return end
		Sender:SendWowMessage(playerTemplate.sender, self.fullPrefix..message, sendInstant, self, callbackValue, playerTemplate)
	end,
	["SendMessage"] = function(self, playerTemplate, message, sendInstant, callbackValue)
		if playerTemplate:GetType() == "BNWoWPlayer" then
			self:SendBNMessage(playerTemplate, message, sendInstant, callbackValue)
		elseif playerTemplate:GetType() == "WoWPlayer" then
			self:SendWoWMessage(playerTemplate, message, sendInstant, callbackValue)
		end
	end,
	["Unregister"] = function(self, notStopMessages)
		self:UnregisterAllCallbackEvents()
		setmetatable(self, nil)
		PrefixRegister[self.prefix] = nil
		if not notStopMessages then
			self.registered = nil
		end
	end,
	-- OVERRIDE
	["OnMessageReceive"] = function(self, sender, message, isBNMessage)

	end,
}
local SENDER_PROTO_MT = { __index = SENDER_PROTO }

--- Registers a new sender
-- @param		prefix				<string/number> the prefix that is used for the messages
-- @param		onMessageReceive	<function> method(prefix, sender, msg, isBNMessage)
-- @param		ignoreGGL			<bool> ignore if player has GGL installed when send a message
function Sender:RegisterReceiver(prefix, onMessageReceive, ignoreGGL)
	if not onMessageReceive then return end
	if PrefixRegister[prefix] then return PrefixRegister[prefix] end
	prefix = tostring(prefix)

	local send = {
		registered = true,
		ignoreGGL = true, --ignoreGGL,
		prefix = prefix,
		fullPrefix = format(MSG_SEND_FORMAT, prefix),
		OnMessageReceive = onMessageReceive
	}

	GGL:AddCallbackHandler(send)

	setmetatable(send, SENDER_PROTO_MT)

	PrefixRegister[prefix] = send
	return send
end

function Sender:UnregisterReceiver(prefix)
	if PrefixRegister[prefix] then
		PrefixRegister[prefix]:Unregister()
		return true
	end
end
-- ###########################
-- Receiving
-- ###########################
local OnReceiveMessage
do
	local msgCache = {}
	local compost = setmetatable({}, {__mode = "k"})
	local function new()
		local t = next(compost)
		if t then
			compost[t]=nil
			for i=#t,3,-1 do	-- faster than pairs loop. don't even nil out 1/2 since they'll be overwritten
				t[i]=nil
			end
			return t
		end

		return {}
	end

	local function ProcessFullMessage(message, sender, isBNMessage)
		--Sender.callbacks:Fire(channel, sender, message)
		local prefix, msg = match(message, MSG_GET_FORMAT)
		if PrefixRegister[prefix] then
			PrefixRegister[prefix]:OnMessageReceive(prefix, sender, msg, isBNMessage)
		end
	end

	local function OnReceiveMultipartFirst(message, channel, sender)
		local key = channel.."\t"..sender

		msgCache[key] = message
	end

	local function OnReceiveMultipartNext(message, channel, sender)
		local key = channel.."\t"..sender
		local oldData = msgCache[key]

		if not oldData then return end

		if type(oldData)~="table" then
			-- ... but what we have is not a table. So make it one. (Pull a composted one if available)
			local t = new()
			t[1] = oldData    -- add old data as first string
			t[2] = message    -- and new message as second string
			msgCache[key] = t    -- and put the table in the spool instead of the old string
		else
			tinsert(oldData, message)
		end
	end

	local function OnReceiveMultipartLast(message, channel, sender, isBNMessage)
		local key = channel.."\t"..sender
		local oldData = msgCache[key]

		if not oldData then return end

		msgCache[key] = nil

		if type(oldData) == "table" then
			-- if we've received a "next", the spooled data will be a table for rapid & garbage-free tconcat
			tinsert(oldData, message)
			--AceComm.callbacks:Fire(prefix, tconcat(oldData, ""), distribution, sender)
			ProcessFullMessage(tconcat(oldData, ""), sender, isBNMessage)
			compost[oldData] = true
		else
			-- if we've only received a "first", the spooled data will still only be a string
			ProcessFullMessage(oldData..message, sender, isBNMessage)
			--AceComm.callbacks:Fire(prefix, olddata..message, distribution, sender)
		end
	end

	function OnReceiveMessage(message, channel, sender, isBNMessage)
		local control, rest = match(message, "^([\001-\009])(.*)")
		if isBNMessage then
			GGL.Templates.BNWoWPlayer:SetOnlineByGameAccountID(sender)
			GGL.Templates.BNWoWPlayer:SetGGLAviableByGameAccountID(sender)
		end
		if control then
			if control==MSG_MULTI_FIRST then
				OnReceiveMultipartFirst(rest, channel, sender)
			elseif control==MSG_MULTI_NEXT then
				OnReceiveMultipartNext(rest, channel, sender)
			elseif control==MSG_MULTI_LAST then
				OnReceiveMultipartLast(rest, channel, sender, isBNMessage)
			elseif control==MSG_ESCAPE then
				ProcessFullMessage(rest, sender, isBNMessage)
			else
				-- unknown control character, ignore SILENTLY (dont warn unnecessarily about future extensions!)
			end
		else
			-- single part
			ProcessFullMessage(message, sender, isBNMessage)
		end
	end
end

-- ###########################
-- Events
-- ###########################
-- https://wow.gamepedia.com/BN_CHAT_MSG_ADDON
function Sender:BN_CHAT_MSG_ADDON(addonPrefix, message, channel, senderPresenceID)
	if addonPrefix == ADDON_CHANNEL and senderPresenceID ~= PLAYER_BN:GetPlayerSender() then
		OnReceiveMessage(message, channel, senderPresenceID, true)
	end
end

-- https://wow.gamepedia.com/CHAT_MSG_ADDON
function Sender:CHAT_MSG_ADDON(addonPrefix, message, channel, sender, target, zoneChannelID, localID, name, instanceID)
	if addonPrefix == ADDON_CHANNEL and sender ~= PLAYER_WOW:GetPlayerSender() then
		OnReceiveMessage(message, channel, sender, false)
	end
end